@@ -368,7 +368,7 @@ traverse_components_xpath_for_icons (GsApp *app,
 			icons = xb_node_query (component, "icon", 0, NULL);
 			traverse_component_icons (app, component, icons);
 
-			if (try_with_launchable && gs_app_get_icons (app) == NULL) {
+			if (try_with_launchable && !gs_app_has_icons (app)) {
 				const gchar *launchable_id = xb_node_query_text (component, "launchable[@type='desktop-id']", NULL);
 				if (launchable_id != NULL) {
 					g_autofree gchar *xpath2 = NULL;
@@ -396,7 +396,7 @@ gs_appstream_refine_icon (GsApp *app,
 	g_clear_pointer (&icons, g_ptr_array_unref);
 
 	/* If no icon found, try to inherit the icon from the .desktop file */
-	if (gs_app_get_icons (app) == NULL) {
+	if (!gs_app_has_icons (app)) {
 		g_autofree gchar *xpath = NULL;
 		const gchar *launchable_id = xb_node_query_text (component, "launchable[@type='desktop-id']", NULL);
 		if (launchable_id != NULL) {
@@ -1160,6 +1160,12 @@ gs_appstream_refine_app (GsPlugin *plugin,
 			gs_app_remove_quirk (app, GS_APP_QUIRK_HIDE_EVERYWHERE);
 	}
 
+	tmp = gs_app_get_metadata_item (app, "flathub::verification::verified");
+	if (g_strcmp0 (tmp, "true") == 0)
+		gs_app_add_quirk (app, GS_APP_QUIRK_DEVELOPER_VERIFIED);
+	else
+		gs_app_remove_quirk (app, GS_APP_QUIRK_DEVELOPER_VERIFIED);
+
 	/* try to detect old-style AppStream 'override'
 	 * files without the merge attribute */
 	if (xb_node_query_text (component, "name", NULL) == NULL &&
@@ -1267,7 +1273,7 @@ gs_appstream_refine_app (GsPlugin *plugin,
 
 	/* set icon */
 	if ((refine_flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON) > 0 &&
-	    gs_app_get_icons (app) == NULL)
+	    !gs_app_has_icons (app))
 		gs_appstream_refine_icon (app, silo, component);
 
 	/* set categories */
@@ -1619,15 +1625,27 @@ gs_appstream_search (GsPlugin *plugin,
 		     GError **error)
 {
 	const Query queries[] = {
+		#ifdef HAVE_AS_SEARCH_TOKEN_MATCH_MEDIATYPE
+		{ AS_SEARCH_TOKEN_MATCH_MEDIATYPE,	"mimetypes/mimetype[text()~=stem(?)]" },
+		#else
 		{ AS_SEARCH_TOKEN_MATCH_MIMETYPE,	"mimetypes/mimetype[text()~=stem(?)]" },
+		#endif
+		/* Search once with a tokenize-and-casefold operator (`~=`) to support casefolded
+		 * full-text search, then again using substring matching (`contains()`), to
+		 * support prefix matching. Only do the prefix matches on a few fields, and at a
+		 * lower priority, otherwise things will get confusing.
+		 * 
+		 * See https://gitlab.gnome.org/GNOME/gnome-software/-/issues/2277 */
 		{ AS_SEARCH_TOKEN_MATCH_PKGNAME,	"pkgname[text()~=stem(?)]" },
+		{ AS_SEARCH_TOKEN_MATCH_PKGNAME / 2,	"pkgname[contains(text(),stem(?))]" },
 		{ AS_SEARCH_TOKEN_MATCH_SUMMARY,	"summary[text()~=stem(?)]" },
-		{ AS_SEARCH_TOKEN_MATCH_NAME,	"name[text()~=stem(?)]" },
+		{ AS_SEARCH_TOKEN_MATCH_NAME,		"name[text()~=stem(?)]" },
+		{ AS_SEARCH_TOKEN_MATCH_NAME / 2,	"name[contains(text(),stem(?))]" },
 		{ AS_SEARCH_TOKEN_MATCH_KEYWORD,	"keywords/keyword[text()~=stem(?)]" },
-		{ AS_SEARCH_TOKEN_MATCH_ID,	"id[text()~=stem(?)]" },
-		{ AS_SEARCH_TOKEN_MATCH_ID,	"launchable[text()~=stem(?)]" },
-		{ AS_SEARCH_TOKEN_MATCH_ORIGIN,	"../components[@origin~=stem(?)]" },
-		{ AS_SEARCH_TOKEN_MATCH_NONE,	NULL }
+		{ AS_SEARCH_TOKEN_MATCH_ID,		"id[text()~=stem(?)]" },
+		{ AS_SEARCH_TOKEN_MATCH_ID,		"launchable[text()~=stem(?)]" },
+		{ AS_SEARCH_TOKEN_MATCH_ORIGIN,		"../components[@origin~=stem(?)]" },
+		{ AS_SEARCH_TOKEN_MATCH_NONE,		NULL }
 	};
 
 	return gs_appstream_do_search (plugin, silo, values, queries, list, cancellable, error);
@@ -1679,11 +1697,11 @@ gs_appstream_add_category_apps (GsPlugin *plugin,
 
 		/* generate query */
 		if (g_strv_length (split) == 1) {
-			xpath = g_strdup_printf ("components/component/categories/"
+			xpath = g_strdup_printf ("components/component[not(@merge)]/categories/"
 						 "category[text()='%s']/../..",
 						 split[0]);
 		} else if (g_strv_length (split) == 2) {
-			xpath = g_strdup_printf ("components/component/categories/"
+			xpath = g_strdup_printf ("components/component[not(@merge)]/categories/"
 						 "category[text()='%s']/../"
 						 "category[text()='%s']/../..",
 						 split[0], split[1]);
@@ -1725,11 +1743,11 @@ gs_appstream_count_component_for_groups (XbSilo      *silo,
 	g_autoptr(GError) error_local = NULL;
 
 	if (g_strv_length (split) == 1) { /* "all" group for a parent category */
-		xpath = g_strdup_printf ("components/component/categories/"
+		xpath = g_strdup_printf ("components/component[not(@merge)]/categories/"
 					 "category[text()='%s']/../..",
 					 split[0]);
 	} else if (g_strv_length (split) == 2) {
-		xpath = g_strdup_printf ("components/component/categories/"
+		xpath = g_strdup_printf ("components/component[not(@merge)]/categories/"
 					 "category[text()='%s']/../"
 					 "category[text()='%s']/../..",
 					 split[0], split[1]);
@@ -1863,7 +1881,7 @@ gs_appstream_add_recent (GsPlugin *plugin,
 			 GCancellable *cancellable,
 			 GError **error)
 {
-	guint64 now = (guint64) g_get_real_time () / G_USEC_PER_SEC;
+	guint64 now = (guint64) g_get_real_time () / G_USEC_PER_SEC, max_future_timestamp;
 	g_autofree gchar *xpath = NULL;
 	g_autoptr(GError) error_local = NULL;
 	g_autoptr(GPtrArray) array = NULL;
@@ -1883,17 +1901,22 @@ gs_appstream_add_recent (GsPlugin *plugin,
 		g_propagate_error (error, g_steal_pointer (&error_local));
 		return FALSE;
 	}
+	/* This is to cover mistakes when the release date is set in the future,
+	   to not have it picked for too long. */
+	max_future_timestamp = now + (3 * 24 * 60 * 60);
 	for (guint i = 0; i < array->len; i++) {
 		XbNode *component = g_ptr_array_index (array, i);
-		g_autoptr(GsApp) app = gs_appstream_create_app (plugin, silo, component, error);
-		guint64 timestamp;
-		if (app == NULL)
-			return FALSE;
+		g_autoptr(GsApp) app = NULL;
+		guint64 timestamp = component_get_release_timestamp (component);
 		/* set the release date */
-		timestamp = component_get_release_timestamp (component);
-		if (timestamp != G_MAXUINT64)
+		if (timestamp != G_MAXUINT64 && timestamp < max_future_timestamp) {
+			app = gs_appstream_create_app (plugin, silo, component, error);
+			if (app == NULL)
+				return FALSE;
+
 			gs_app_set_release_date (app, timestamp);
-		gs_app_list_add (list, app);
+			gs_app_list_add (list, app);
+		}
 	}
 	return TRUE;
 }
